<?php

declare(strict_types=1);

namespace Drupal\generic_templates\Form;

use Drupal\Core\Theme\Registry;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityTypeRepositoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Generic Templates settings for this site.
 */
class GenericTemplatesSettings extends ConfigFormBase
{
  public function __construct(
    protected readonly EntityTypeManagerInterface $entityTypeManager,
    protected readonly EntityTypeRepositoryInterface $entityTypeRepository,
    protected readonly Registry $themeRegistry
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container)
  {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('entity_type.repository'),
      $container->get('theme.registry')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string
  {
    return 'generic_templates_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array
  {
    return ['generic_templates.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array
  {
    $config = $this->config('generic_templates.settings');
    $options = $this->entityTypeRepository->getEntityTypeLabels(TRUE)['Content'];

    $form['types'] = [
      '#type' => 'details',
      '#title' => t('Generic Types'),
      '#open' => TRUE,
      'types' => [
        '#type' => 'checkboxes',
        '#description' => t("Use a generic template with minimal CSS for selected Entities. Thats good for theming based on 'compositional CSS selectors'."),
        '#options' => $options,
        '#default_value' => $config->get('types')
      ]
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void
  {
    $config = $this->config('generic_templates.settings');
    $types = $form_state->getValue('types');
    $config->set('types', array_filter($types));
    
    $config->save();

    parent::submitForm($form, $form_state);

    // @todo clear render cache tags for every changed entity_type
    // $changed = array_xor($selected, );
    $this->themeRegistry->reset();
  }
}
